#include <cstdio>
#include <iostream>
#include <cassert>
#include <vector>
#include <map>
#include <string>
#include <algorithm>
#include <sstream>
#include "testlib.h"

using namespace std;

int cur_test = -1;
const int GROUPS_CNT = 6;
const int MAX_K[GROUPS_CNT] = {1e5, 1e5, 2, 10, 1e5, 1e5};
const int MAX_N[GROUPS_CNT] = {1e6, 1000, 1000, 1000, 1000, 1e5};


string itoa(int x) 
{
    stringstream ss;
    string s;
    ss << x;
    ss >> s;
    return s;
}

int atoi(string s) 
{
    stringstream ss;
    int x;
    ss << s;
    ss >> x;
    return x;
}

void write_test(int m, int k, vector<int> test) 
{
    startTest(cur_test++);
    cerr << cur_test << endl;
    cout << test.size() << ' ' << m << ' ' << k << endl;
    for (int i = 0; i < (int)test.size() - 1; i++)
        cout << test[i] <<  ' ';
    cout << test.back() << endl;
}

void gen_rand(int n, int m, int k)
{
    vector<int> test(n);
    for (int i = 0; i < n; i++)
        test[i] = rnd.next(1, k);
    write_test(m, k, test);
}

void gen_1tok(int n, int m, int k)
{
    vector<int> test(n);
    for (int i = 0; i < n; i++)
        test[i] = i % k + 1;
    write_test(m, k, test);
}

void gen_1tok_shuffle(int n, int m, int k)
{
    vector<int> test(n);
    for (int i = 0; i < n; i++)
        test[i] = i % k + 1;
    shuffle(test.begin(), test.end());
    write_test(m, k, test);
}

void gen_1tok_non(int n, int m, int k, int it)
{
    vector<int> test(n);
    for (int i = 0; i < n; i++)
        test[i] = i % k + 1;
    for (int i = 0; i < it; i++)
        test[rnd.next(n)] = rnd.next(1, k);
    write_test(m, k, test);
}


int main(int argc, char * argv[]) 
{
    registerGen(argc, argv, 1);
    cur_test = atoi(argv[1]);
    //1
    gen_rand(15, 15, 3);
    gen_1tok(10, 10, 1);
    gen_1tok_shuffle(25, 25, 12);
    gen_1tok_non(30, 30, 6, 5);

    gen_rand(MAX_N[1], MAX_N[1], MAX_N[1]);
    gen_rand(MAX_N[1], MAX_N[1], MAX_K[1]);
    gen_rand(MAX_N[1], MAX_N[1], 5);
    gen_rand(MAX_N[1], MAX_N[1], 10);
    gen_1tok(MAX_N[1], MAX_N[1], 13);
    gen_1tok_shuffle(MAX_N[1], MAX_N[1], 7);
    gen_1tok_non(MAX_N[1], MAX_N[1], 25, 131);
    cerr << "NEXT:" << endl;
    //2
    gen_rand(15, 10, 2);
    gen_1tok(10, 6, 2);
    gen_1tok_shuffle(25, 15, 2);
    gen_1tok_non(30, 13, 2, 6);

    gen_rand(MAX_N[2], rnd.next(1, MAX_N[2]), MAX_K[2]);
    gen_rand(MAX_N[2], rnd.next(1, MAX_N[2]), MAX_K[2]);
    gen_1tok(MAX_N[2], rnd.next(1, MAX_N[2]), 2);
    gen_1tok_shuffle(MAX_N[2], rnd.next(1, MAX_N[2]), 2);
    gen_1tok_non(MAX_N[2], rnd.next(1, MAX_N[2]), 2, 100);
    cerr << "NEXT:" << endl;
    //3
    gen_rand(10, 5, 3);
    gen_1tok(21, 10, 7);
    gen_1tok_shuffle(20, 10, 5);
    gen_1tok_non(20, 10, 5, 7);
    gen_rand(MAX_N[3], rnd.next(1, MAX_N[3]), rnd.next(1, MAX_K[3]));
    gen_rand(MAX_N[3], rnd.next(1, MAX_N[3]), rnd.next(3, MAX_K[3]));
    gen_1tok(MAX_N[3], rnd.next(1, MAX_N[3]), rnd.next(5, MAX_K[3]));
    gen_1tok_shuffle(MAX_N[3], rnd.next(1, MAX_N[3]), rnd.next(1, MAX_K[3]));
    gen_1tok_non(MAX_N[3], rnd.next(1, MAX_N[3]), rnd.next(3, MAX_K[3]), 50);
    cerr << "NEXT:" << endl;
    //4-5
    for (int i = 4; i <= 5; i++)
    {
        gen_rand(MAX_N[i], MAX_N[i] / 2, MAX_K[i]);
        int m = rnd.next(1, MAX_N[i]);
        gen_rand(MAX_N[i], m, rnd.next(1, m));
        m = rnd.next(MAX_N[i] / 2, MAX_N[i]); 
        gen_rand(MAX_N[i], m, rnd.next(3, m));
        m = rnd.next(1, MAX_N[i]);
        gen_1tok(MAX_N[i], m, rnd.next(5, m));
        m = rnd.next(MAX_N[i] / 2, MAX_N[i]);
        gen_1tok_shuffle(MAX_N[i], m, rnd.next(1, m));
        m = rnd.next(1, MAX_N[i]);
        gen_1tok_non(MAX_N[i], m, rnd.next(3, m), rnd.next(1, MAX_N[i] / 3));
        cerr << "NEXT:" << endl;
    }
    //extra 5
    gen_rand(MAX_N[5], rnd.next(MAX_N[5] / 2, MAX_N[5]), rnd.next(1, MAX_K[5]));
    gen_1tok(MAX_N[5], MAX_N[5], MAX_K[5]);
    gen_1tok_shuffle(MAX_N[5], MAX_N[5], MAX_K[5]);
    gen_1tok_non(MAX_N[5], rnd.next(MAX_N[5] / 2, MAX_N[5]), MAX_K[5], MAX_N[5] / 10);
    
    return 0;
}